package org.apidesign.apifest08.test;

import junit.framework.TestCase;
import org.apidesign.apifest08.currency.Convertor;
import org.apidesign.apifest08.currency.ConvertorFactory;
import org.apidesign.apifest08.currency.CurrencyRate;
import org.apidesign.apifest08.currency.Rate;

/** The exchange rates are not always the same. They are changing. Day by day,
 * hour by hour, minute by minute. For every bank it is important to always
 * have the actual exchange rate available in the system. That is why let's
 * create a pluggable convertor that will always have up to date value of its
 * exchange rate.
 * <p>
 * The quest for today is to allow 3rd party developer to write a convertor
 * that adjusts its exchange rate everytime it is queried. This convertor is
 * written by independent vendor, the vendor knows only your Convertor API,
 * he does not know how the whole system looks and how the convertor is supposed
 * to be used.
 */
public class Task3Test extends TestCase {
    public Task3Test(String testName) {
        super(testName);
    }

    @Override
    protected void setUp() throws Exception {
    }

    @Override
    protected void tearDown() throws Exception {
    }

    // Backward compatibly enhance your existing API to support following
    // usecases:
    //


    /** Without knowing anything about the surrounding system, write an
     * implementation of convertor that will return different rates everytime
     * it is queried. Convert USD to CZK and vice versa. Start with the rate of
     * 1USD = 16CZK and adjust it in favor of CZK by 0.01 CZK with every query.
     * As soon as you reach 1USD = 15CZK adjust it by 0.01 CZK in favor of USD
     * until you reach 1USD = 16CZK
     *
     * @return new instance of "online" USD and CZK convertor starting with rate 1USD = 16CZK
     */
    public static Convertor createOnlineCZKUSDConvertor() {
        // initial rate: 1USD = 16CZK
        // 2nd query 1USD = 15.99CZK
        // 3rd query 1USD = 15.98CZK
        // until 1USD = 15.00CZK
        // then 1USD = 15.01CZK
        // then 1USD = 15.02CZK
        // and so on and on up to 1USD = 16CZK
        // and then another round to 15, etc.
        CurrencyRate onlineCurrencyRate = new CurrencyRate() {
            private int usdAmount = 100;
            private int czkAmount = 1600;
            private boolean up = false;
            public String getCurrency1() {
                return "USD";
            }

            public String getCurrency2() {
                return "CZK";
            }

            public Rate getRate() { //return Rate according to online status
                Rate rate = new Rate(usdAmount, czkAmount);
                if (up) {
                    if (czkAmount < 1600) {
                        czkAmount++;
                    } else {
                        up = false;
                        czkAmount--;
                    }
                } else { //down
                    if (czkAmount > 1500) {
                        czkAmount--;
                    } else {
                        up = true;
                        czkAmount++;
                    }
                }
                return rate;
            }
        };

        return ConvertorFactory.newInstance().createConvertor(onlineCurrencyRate);
    }

    public void testFewQueriesForOnlineConvertor() {
//        if (Boolean.getBoolean("ignore.failing")) {
//            // implement me!
//            return;
//        }

        Convertor c = createOnlineCZKUSDConvertor();
        doFewQueriesForOnlineConvertor(c);
    }

    static void doFewQueriesForOnlineConvertor(Convertor c) {
        // convert $5 to CZK using c:
        //assertEquals("Result is 80 CZK");
        assertEquals("Result is 80 CZK", 80.0, c.convert("USD", "CZK", 5), 0.001);

        // convert $8 to CZK using c:
        //assertEquals("Result is 127.92 CZK");
        assertEquals("Result is 127.92 CZK", 127.92, c.convert("USD", "CZK", 8), 0.001);

        // convert $1 to CZK using c:
        //assertEquals("Result is 15.98 CZK");
        assertEquals("Result is 15.98 CZK", 15.98, c.convert("USD", "CZK", 1), 0.001);

        // convert 15.97CZK to USD using c:
        //assertEquals("Result is 1$");
        assertEquals("Result is 1$", 1.0, c.convert("CZK", "USD", 15.97), 0.001);
        
    }

    /** Join the convertors and show they behave sane.
     */
    public void testOnlineConvertorComposition() throws Exception {
        Convertor c = Task2Test.merge(
            createOnlineCZKUSDConvertor(),
            Task1Test.createSKKtoCZK()
        );

        // convert 16CZK to SKK using c:
        // assertEquals("Result is 20 SKK");
        assertEquals("Result is 20 SKK", 20.0, c.convert("CZK", "SKK", 16), 0.001);

        // convert 500SKK to CZK using c:
        // assertEquals("Result is 400 CZK");
        assertEquals("Result is 400 CZK", 400.0, c.convert("SKK", "CZK", 500), 0.001);

        doFewQueriesForOnlineConvertor(c);
    }
}
